==> 1.0.0.tar.gz: lib/src/sub_types/TypeAlias.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../lists/TypeParameters.dart';
import '../main/MetaData.dart';
import '../core/Identifier.dart';
import '../core/Type.dart';
import 'FormalParameterPart.dart';
import 'Identifiers.dart';
import 'Types.dart';

/*
<typeAlias> ::=
<metadata> typedef <typeIdentifier> <typeParameters>? '=' <functionType> ';'
| <metadata> typedef <functionTypeAlias>

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 179
*/
class TypeAlias extends ParserType {
  int caseNum;
  MetaData metaData;
  TypeIdentifier typeIdentifier;
  TypeParameters typeParameters;
  FunctionType functionType;
  FunctionTypeAlias functionTypeAlias;

  TypeAlias(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in TypeAlias: ALL FAILED');
  }


  /* <metadata> typedef <typeIdentifier> <typeParameters>? '=' <functionType> ';' */
  void _case0(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'typedef');
    lexer.advance();

    typeIdentifier = TypeIdentifier(lexer);

    int snapShot = lexer.currentIndex;
    try {
      typeParameters = TypeParameters(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.EQUALS);
    lexer.advance();

    functionType = FunctionType(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    caseNum = 0;
  }
  
  /* <metadata> typedef <functionTypeAlias> */
  void _case1(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'typedef');
    lexer.advance();

    functionTypeAlias = FunctionTypeAlias(lexer);

    caseNum = 1;
  }
  
  /* <metadata> typedef <typeIdentifier> <typeParameters>? '=' <functionType> ';' */
  String toString() {
    switch (caseNum) {
      case 0:
        return '${metaData}typedef $typeIdentifier ' +
               (typeParameters != null ? '$typeParameters ' : '') +
               '= $functionType;';
        break;
      case 1:
        return '${metaData}typedef $functionTypeAlias';
        break;
    }
    return '';
  }

}

/* <functionTypeAlias> ::= <functionPrefix> <formalParameterPart> ';' */
class FunctionTypeAlias extends ParserType {
  FunctionPrefix functionPrefix;
  FormalParameterPart formalParameterPart;

  FunctionTypeAlias(Lexer lexer) {
    functionPrefix = FunctionPrefix(lexer);
    formalParameterPart = FormalParameterPart(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return '$functionPrefix $formalParameterPart;';
  }

}

/* <functionPrefix> ::= <type>? <identifier> */
class FunctionPrefix extends ParserType {
  Type type;
  Identifier identifier;

  FunctionPrefix(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      type = Type(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    identifier = Identifier(lexer);
  }

  String toString() {
    return (type != null ? '$type ' : '') + '$identifier';
  }

}

==> 1.1.1.tar.gz: lib/src/sub_types/TypeAlias.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../lists/TypeParameters.dart';
import '../main/MetaData.dart';
import '../core/Identifier.dart';
import '../core/Type.dart';
import 'FormalParameterPart.dart';
import 'Identifiers.dart';
import 'Types.dart';

/// <typeAlias> ::=
/// <metadata> typedef <typeIdentifier> <typeParameters>? '=' <functionType> ';'
/// | <metadata> typedef <functionTypeAlias>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 179
class TypeAlias extends ParserType {
  int caseNum;
  MetaData metaData;
  TypeIdentifier typeIdentifier;
  TypeParameters typeParameters;
  FunctionType functionType;
  FunctionTypeAlias functionTypeAlias;

  TypeAlias(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType('Tried all options in TypeAlias: ALL FAILED');
    }
  }

  /// <metadata> typedef <typeIdentifier> <typeParameters>? '=' <functionType> ';'
  void _case0(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'typedef');
    lexer.advance();

    typeIdentifier = TypeIdentifier(lexer);

    int snapShot = lexer.currentIndex;
    try {
      typeParameters = TypeParameters(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.EQUALS);
    lexer.advance();

    functionType = FunctionType(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    caseNum = 0;
  }

  /// <metadata> typedef <functionTypeAlias>
  void _case1(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'typedef');
    lexer.advance();

    functionTypeAlias = FunctionTypeAlias(lexer);

    caseNum = 1;
  }

  /// <metadata> typedef <typeIdentifier> <typeParameters>? '=' <functionType> ';'
  String toString() {
    switch (caseNum) {
      case 0:
        return '${metaData}typedef $typeIdentifier ' +
            (typeParameters != null ? '$typeParameters ' : '') +
            '= $functionType;';
        break;
      case 1:
        return '${metaData}typedef $functionTypeAlias';
        break;
    }
    return '';
  }
}

/// <functionTypeAlias> ::= <functionPrefix> <formalParameterPart> ';'
class FunctionTypeAlias extends ParserType {
  FunctionPrefix functionPrefix;
  FormalParameterPart formalParameterPart;

  FunctionTypeAlias(Lexer lexer) {
    functionPrefix = FunctionPrefix(lexer);
    formalParameterPart = FormalParameterPart(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return '$functionPrefix $formalParameterPart;';
  }
}

/// <functionPrefix> ::= <type>? <identifier>
class FunctionPrefix extends ParserType {
  Type type;
  Identifier identifier;

  FunctionPrefix(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      type = Type(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    identifier = Identifier(lexer);
  }

  String toString() {
    return (type != null ? '$type ' : '') + '$identifier';
  }
}
